/*
 * <aisl/callback.h>
 *
 * Copyright (c) 2017 by Löwenware Ltd.
 *
 * Project homepage: https://lowenware.com/aisl/
 *
 */

#ifndef AISL_CALLBACK_H_E99458C9_CCAA_4CAC_97F8_41E8869EE26F
#define AISL_CALLBACK_H_E99458C9_CCAA_4CAC_97F8_41E8869EE26F

#include <stdbool.h>
#include <stdarg.h>
#include <arpa/inet.h>

#include <aisl/server.h>
#include <aisl/client.h>
#include <aisl/stream.h>
#include <aisl/http.h>

#define AISL_CALLBACK(x) ((aisl_callback_t) x)


/* void-type callback */
typedef void
(* aisl_callback_t) (void);


/* real type callbacks */

typedef bool
(*aisl_server_open_t)( aisl_server_t server, int flags );

typedef bool
(*aisl_server_error_t)( aisl_server_t server, int flags, const char * details );

typedef bool
(*aisl_client_connect_t)( aisl_server_t server, aisl_client_t client );

typedef bool
(*aisl_client_disconnect_t)( aisl_server_t server, aisl_client_t client );

typedef bool
(*aisl_client_timeout_t)( aisl_server_t server, aisl_client_t client );

typedef bool
(*aisl_stream_open_t)( aisl_stream_t        stream,
                       aisl_http_method_t   method,
                       const char         * path,
                       const char         * query );

typedef bool
(*aisl_stream_header_t)( aisl_stream_t    stream,
                         const char     * key,
                         const char     * val );

typedef bool
(*aisl_stream_input_t)( aisl_stream_t stream, char * data, int len );

typedef bool
(*aisl_stream_request_t)( aisl_stream_t stream );

typedef bool
(*aisl_stream_output_t)( aisl_stream_t stream, uint32_t buffer_space );

typedef bool
(*aisl_stream_close_t)( aisl_stream_t stream );

typedef bool
(*aisl_stream_error_t)( aisl_stream_t stream, const char * details );

typedef bool
(*aisl_custom_event_t)( void * source, va_list vl );

#endif /* !AISL_CALLBACK_H */
